#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Alan Viars

import os, sys, string, json, csv, time, uuid
from collections import OrderedDict
from datetime import datetime






def new_pjson_stub():

    ps = OrderedDict()

    #ProviderJSON stub
    ps['enumeration_type']=""
    ps['number'] =""
    ps['last_updated_epoch'] =""
    ps['created_epoch'] =""
    ps["basic"] = OrderedDict()
    ps["basic"]["status"]= ""
    ps["basic"]["name_prefix"]= ""
    ps["basic"]["first_name"]= ""
    ps["basic"]["last_name"]= ""
    ps["basic"]["middle_name"]= ""
    ps["basic"]["name_suffix"]= ""
    ps["basic"]["credential"]= ""
    ps["basic"]["doing_business_as"]= ""
    ps["basic"]["sole_proprietor"]= ""
    ps["basic"]["other_first_name_1"]= ""
    ps["basic"]["other_first_name_2"]= ""
    ps["basic"]["other_last_name_1"]= ""
    ps["basic"]["other_last_name_2"]= ""
    ps["basic"]["other_middle_name_1"]= ""
    ps["basic"]["other_middle_name_2"]= ""
    ps["basic"]["other_name_code_1"]= ""
    ps["basic"]["other_name_code_2"]= ""
    ps["basic"]["other_name_credential_1"]= ""
    ps["basic"]["other_name_credential_2"]= ""
    ps["basic"]["other_name_prefix_1"]= ""
    ps["basic"]["other_name_prefix_2"]= ""
    ps["basic"]["other_name_suffix_1"]= ""
    ps["basic"]["other_name_suffix_2"]= ""
    ps["basic"]["organization_name"]= ""
    ps["basic"]["organization_other_name"]= ""
    ps["basic"]["organization_other_name_code"]= ""
    ps["basic"]["organizational_subpart"]= ""
    ps["basic"]["ssn"]= ""
    ps["basic"]["ein"]= ""
    ps["basic"]["itin"]= ""
    ps["basic"]["gender"]= ""
    ps["basic"]["date_of_birth"]= ""
    ps["basic"]["state_of_birth"]= ""
    ps["basic"]["country_of_birth"]= ""
    ps["basic"]["number"]= ""
    ps["basic"]["initial_enumeration_date"]= ""
    ps["basic"]["enumeration_date"]= ""
    ps["basic"]["last_updated"]= ""
    ps["basic"]["date_of_death"]= ""
    ps["basic"]["reactivation_date"]= ""
    ps["basic"]["contact_method"]= "E"
    ps["basic"]["deactivated_details"]= ""
    ps["basic"]["deactivation_date"]= "None"
    ps["basic"]["deactivation_reason_code"]= ""
    ps["basic"]["decativation_note"]= ""
    ps["basic"]["deceased_notes"]= ""
    ps["basic"]["parent_organization"]= ""
    ps["basic"]["parent_organization_ein"]= ""
    ps["basic"]["parent_organization_legal_business_name"]= ""
    ps["basic"]["recativation_note"]= ""
    ps["basic"]["comments"]= ""
    ps["basic"]["authorized_official_credential"]= ""
    ps["basic"]["authorized_official_email"]= ""
    ps["basic"]["authorized_official_first_name"]= ""
    ps["basic"]["authorized_official_last_name"]= ""
    ps["basic"]["authorized_official_middle_name"]= ""
    ps["basic"]["authorized_official_prefix"]= ""
    ps["basic"]["authorized_official_suffix"]= ""
    ps["basic"]["authorized_official_telephone_number"]= ""
    ps["basic"]["authorized_official_telephone_extension"]= ""
    ps["basic"]["authorized_official_title"]= ""
    ps["basic"]["authorized_official_title_or_position"]= ""
    ps["basic"]["contact_person_credential"]= ""
    ps["basic"]["contact_person_email"]= ""
    ps["basic"]["contact_person_first_name"]= ""
    ps["basic"]["contact_person_last_name"]= ""
    ps["basic"]["contact_person_middle_name"]= ""
    ps["basic"]["contact_person_prefix"]= ""
    ps["basic"]["contact_person_suffix"]= ""
    ps["basic"]["contact_person_telephone_extension"]= ""
    ps["basic"]["contact_person_telephone_number"]= ""
    ps["basic"]["contact_person_title"]= ""
    ps["basic"]["contact_person_title_or_position"]= ""
    ps["addresses"] = []
    ps["taxonomies"] = []
    ps["licenses"] = []
    ps["identifiers"] = []
    return ps




def gao_csv2pjson(csvfile, output_dir):
    
    """Return a response_dict with summary of gao_csv2pjson transaction."""


    process_start_time = time.time()

    pdir = 1


    #make the output dir
    try:
        os.mkdir(output_dir)
    except:
        pass


    response_dict = OrderedDict()
    fh = open(csvfile, 'rb')
    csvhandle = csv.reader(fh, delimiter=',')
    rowindex = 0
    po_count = 0
    error_list = []

    for row in csvhandle :

        if rowindex==0:

            rowindex += 1
            column_headers = row

            cleaned_headers = []
            for c in column_headers:
                c= c.replace(".", "")
                c= c.replace("(", "")
                c= c.replace(")", "")
                c =c.replace("$", "-")
                c =c.replace(" ", "_")
                cleaned_headers.append(c)
        else:

            #If the records is not redacted
            if row[1]:
                zip_record = zip(cleaned_headers, row)
                record = dict(zip(cleaned_headers, row))

                #get rid of blanks
                no_blank_zip = []
                for i in zip_record:
                    if i[1]:
                        no_blank_zip.append(i)

                #start our object off with a stub.
                p = new_pjson_stub()


                if row[1] == "1":
                    p["enumeration_type"] = "NPI-1"
                if row[1] == "2":
                    p["enumeration_type"] = "NPI-2"

                #determine the status
                if row[0]:
                    p["number"]             = int(row[0])

                #Load basic ----------------------------------------

                #if deactivated date and no reactivation date
                if row[43] and not row[44]:
                    p["basic"]["status"]= "D"
                    month  = row[43][0:2]
                    day    = row[43][3:5]
                    year   = row[43][6:10]
                    #deactivation date
                    p["basic"]["deactivation_date"] = "%s-%s-%s" % (year, month, day)
                else:
                    p["basic"]["status"]= "A"
                
                #reactivation date
                if row[44]:
                    month  = row[44][0:2]
                    day    = row[44][3:5]
                    year   = row[44][6:10]
                    p["basic"]["reactivation_date"] = "%s-%s-%s" % (year, month, day)
                

                #PII
                p["basic"]["ssn"] =  row[3]
                p["basic"]["itin"] = row[4]
                p["basic"]["ein"] =  row[5]
                p["basic"]["state_of_birth"] = row[47]
                p["basic"]["country_of_birth"] = row[48]

                if row[46]:
                    month  = row[46][0:2]
                    day    = row[46][3:5]
                    year   = row[46][6:10]
                    p["basic"]["date_of_birth"] = "%s-%s-%s" % (year, month, day)

                p['basic']["organization_parent_ein"]  =  row[331]


                #Provider name
                p["basic"]["name_prefix"]       = row[10].capitalize()
                p["basic"]['first_name']        = row[8].capitalize()
                p["basic"]["middle_name"]       = row[9].capitalize()
                p["basic"]['last_name']         = row[7].capitalize()
                p["basic"]["name_suffix"]       = row[11].capitalize()
                p["basic"]["credential"]        = row[12]
                p["basic"]["gender"]            = row[45]

                if row[307] in (True, "Y", "y", "YES"):
                    p["basic"]["sole_proprietor"]       = "YES"
                else:
                    p["basic"]["sole_proprietor"]       = "NO"

                p["basic"]["parent_organization_ein"]   = row[310]

                #provider_organization
                p["basic"]["organization_name"]         = row[13]
                p["basic"]["parent_organization_lbn"]   = row[309]

                #other name
                p["basic"]["other_organization_name"]   = row[13]
                p["basic"]["other_organization_code"]   = row[14]


                p["basic"]["other_last_name_1"]     = row[15].capitalize()
                p["basic"]["other_first_name_1"]    = row[16].capitalize()
                p["basic"]["other_middle_name_1"]   = row[17].capitalize()
                p["basic"]["other_name_prefix_1"]   = row[18].capitalize()
                p["basic"]["other_name_suffix_1"]   = row[19].capitalize()
                p["basic"]["other_name_credential_1"] = row[20]
                p["basic"]["other_name_code_1"]       = row[21]

                p["basic"]["enumeration_date"]          = row[40]
                
                if p["basic"]["enumeration_date"]:
                    p['created_epoch'] = int(datetime(int(p["basic"]["enumeration_date"][6:10]),
                                                           int(p["basic"]["enumeration_date"][0:2]),
                                                           int(p["basic"]["enumeration_date"][3:5]),
                                                           0, 0).strftime('%s'))
                
                
                p["basic"]["last_updated"]              = row[41]
                if p["basic"]["last_updated"]:
                    p['last_updated_epoch'] = int(datetime(int(p["basic"]["last_updated"][6:10]),
                                                           int(p["basic"]["last_updated"][0:2]),
                                                           int(p["basic"]["last_updated"][3:5]),
                                                           0, 0).strftime('%s'))
                
                
                p["basic"]["deactivation_date"]         = row[43]
                p["basic"]["reactivation_date"]         = row[44]
                p["basic"]["deactivation_reason_code"]  = row[42]


                if row[40]:
                    month = row[40][0:2]
                    day =  row[40][3:5]
                    year = row[40][6:10]
                    p["basic"]["enumeration_date"] = "%s-%s-%s" % (year, month, day)

                if row[41]:
                    month = row[41][0:2]
                    day =  row[41][3:5]
                    year = row[41][6:10]
                    p["basic"]["last_updated"] = "%s-%s-%s" % (year, month, day)

                if row[43]:
                    month = row[43][0:2]
                    day =  row[43][3:5]
                    year = row[43][6:10]
                    p["basic"]["deactivation_date"] = "%s-%s-%s" % (year, month, day)


                if row[44]:
                    month = row[44][0:2]
                    day =  row[44][3:5]
                    year = row[44][6:10]
                    p["basic"]["reactivation_date"] = "%s-%s-%s" % (year, month, day)


                #Contact person
                p["basic"]["contact_person_last_name"]          = row[58]
                p["basic"]["contact_person_first_name"]         = row[59]
                p["basic"]["contact_personl_middle_name"]       = row[60]
                p["basic"]["contact_person_prefix"]             = row[61]
                p["basic"]["contact_person_suffix"]             = row[62]
                p["basic"]["contact_person_credential"]         = row[63]
                p["basic"]["contact_person_title_or_position"]  = row[64]
                p["basic"]["contact_person_telephone_number"]   =  "%s-%s-%s" % (row[65][0:3], row[65][3:6], row[65][6:12])
                p["basic"]["contact_person_telephone_extension"] =  row[66]
                p["basic"]["contact_person_email"]               =  row[67]

                #not included in public file

                #Authorized official
                p["basic"]["authorized_official_last_name"] = row[49]
                p["basic"]["authorized_official_first_name"] = row[50]
                p["basic"]["authorized_official_middle_name"] = row[51]
                p["basic"]["authorized_official_prefix"] = row[52]
                p["basic"]["authorized_official_suffix"] = row[53]
                p["basic"]["authorized_official_credential"] = row[54]
                p["basic"]["authorized_official_title_or_position"] = row[55]

                if row[56]:
                    p["basic"]["authorized_official_telephone_number"] =  "%s-%s-%s" % (row[56][0:3], row[56][3:6], row[56][6:12])
                p["basic"]["authorized_official_telephone_extension"] =  row[57]
                p["basic"]["mode"] = "W"


                clean_basic = OrderedDict()
                for k,v in p["basic"].items():
                    if v:
                        if v.isdigit():
                            clean_basic[k] = int(v)
                        else:
                            clean_basic[k] = v
                    
                p["basic"] = clean_basic

                #Addresses
                #location ----------------------------------------------------
                a = OrderedDict()
                a["country_code"]                    = row[36]
                a["address_purpose"]                 = "LOCATION"

                if a["country_code"] == "US":
                    a["address_type"]                    = "DOM"
                    a["address_1"]                       =  row[31]
                    a["address_2"]                       =  row[32]
                    a["city"]                            =  row[33]
                    a["state"]                           =  row[34]
                    a["zip"]                             =  row[35]
                    
                    if a["zip"].isdigit():
                        a["zip"] = int(a["zip"])
                    
                    a["telephone_number_extension"]      =  row[38]

                    if row[34]:

                        a["us_telephone_number"]         =  "%s-%s-%s" % (row[37][0:3], row[37][3:6], row[37][6:12])

                    if  row[35]:
                        a["us_fax_number"]               =  "%s-%s-%s" % (row[39][0:3], row[39][3:6], row[39][6:12])

                else:
                    a["address_type"]                    = "FGN"
                    a["address_1"]                       =  row[31]
                    a["address_2"]                       =  row[32]
                    a["city"]                            =  row[33]
                    a["foreign_state"]                   =  row[34]
                    a["foreign_postal"]                  =  row[35]
                    
                    if a["foreign_postal"].isdigit():
                        a["foreign_postal"] = int(a["foreign_postal"])
                        
                    a["foreign_telephone_number"]        =  row[34]
                    a["foreign_fax_number"]              =  row[35]
                    a["telephone_number_extension"]      =  row[38]


                p['addresses'].append(a)

                #Mailing address ---
                a = OrderedDict()
                a["country_code"]                    = row[27]
                a["address_purpose"]                 = "MAILING"

                if a["country_code"] == "US":
                    a["address_type"]                    = "DOM"
                    a["address_1"]                       =  row[22]
                    a["address_2"]                       =  row[23]
                    a["city"]                            =  row[24]
                    a["state"]                           =  row[25]
                    a["zip"]                             =  row[26]
                    
                    if a["zip"].isdigit():
                        a["zip"] = int(a["zip"])
                    
                    
                    a["telephone_number_extension"]      =  row[29]
                    if row[26]:
                        a["us_telephone_number"] =  "%s-%s-%s" % (row[28][0:3], row[28][3:6], row[28][6:12])

                    if row[27]:

                        a["us_fax_number"]       =  "%s-%s-%s" % (row[30][0:3], row[30][3:6], row[30][6:12])

                else:
                    a["address_type"]                    = "FGN"
                    a["address_1"]                       =  row[22]
                    a["address_2"]                       =  row[23]
                    a["city"]                            =  row[24]
                    a["foreign_state"]                   =  row[25]
                    a["foreign_postal"]                  =  row[26]
                    
                    if a["foreign_postal"].isdigit():
                        a["foreign_postal"] = int(a["foreign_postal"])
                    
                    a["foreign_telephone_number"]        =  row[28]
                    a["foreign_fax_number"]              =  row[30]


                p['addresses'].append(a)

                #licenses and taxonomies----------------------------------------
                #starting at row 47, 2nd set starts at 51

                #Taxonomy
                taxonomy_code_position = 68
                primary_taxonomy_position = 71
                license_state_position = 70
                license_number_position = 69


                for i in range(1, 15):

                    if row[taxonomy_code_position]:
                        taxonomy = OrderedDict()
                        taxonomy['code'] = row[taxonomy_code_position]
                        if row[primary_taxonomy_position] in ("Y", "True", True, "YES"):
                            taxonomy['primary'] = True
                        else:
                            taxonomy['primary'] = False

                        p['taxonomies'].append(taxonomy)

                     #License
                    if row[license_number_position]:

                        license             = OrderedDict()
                        mlvs                = "%s-UNK-%s" % (row[license_state_position], row[license_number_position])
                        license['code']     = mlvs
                        license['state']    = row[license_state_position]
                        license['type']     = "UNK"
                        license['status']   = "UNK"

                        p['licenses'].append(license)


                    taxonomy_code_position += 4
                    primary_taxonomy_position += 4
                    license_state_position += 4
                    license_number_position += 4

                #identifiers----------------
                #starting at row 107

                identifier_position = 128
                identifier_code_position = 129
                identifier_state_position = 130
                identifier_issuer_position = 131

                for i in range(1, 50):

                    if row[identifier_position]:
                        identifier = OrderedDict()
                        identifier['identifier'] = row[identifier_position]
                        
                        if identifier["identifier"].isdigit():
                            identifier["identifier"] = int(identifier["identifier"])
                        
                        
                        identifier['code'] = row[identifier_code_position]
                        identifier['state'] = row[identifier_state_position]
                        identifier['issuer'] = row[identifier_issuer_position]
                        p['identifiers'].append(identifier)


                    identifier_position += 4
                    identifier_code_position  += 4
                    identifier_state_position  += 4
                    identifier_issuer_position  += 4


                if p["number"]:
                    fn = "%s.json" % (p["number"])
                    subdir = os.path.join(output_dir, str(p["number"])[0:4])
                else:
                    guid = str(uuid.uuid4())
                    if p["enumeration_type"]=="NPI-1" and  p["basic"]["first_name"] and p["basic"]["last_name"]:
                        fn = "%s-%s-%s.json" % (p["basic"]["first_name"], p["basic"]["last_name"], guid[0:8])
                    elif  p["enumeration_type"]=="NPI-2" and p["basic"]["organization_name"]:
                        fn = "%s-%s.json" % (p["basic"]["organization_name"], guid[0:8])
                    else:
                        fn = "%s.json" % (guid)
                    
                    subdir = os.path.join(output_dir, "no-id")

                try:
                    os.mkdir(subdir)
                except:
                    pass


                fp = os.path.join(subdir, fn)
                ofile =  open(fp, 'w')
                ofile.writelines(json.dumps(p, indent =4))
                ofile.close()
                po_count += 1
                if po_count % 100 == 0:
                   out  = "%s files created. Total time: %s seconds." % (po_count ,(time.time() - process_start_time) )
                   print out

                if po_count % 1000 == 0:
                   pdir += 1

                   out  = "%s files created. Total time: %s seconds." % (po_count ,(time.time() - process_start_time) )
                   print out


            rowindex += 1


        if error_list:
                response_dict['num_files_created']=rowindex-1
                response_dict['num_file_errors']=len(error_list)
                response_dict['errors']=error_list
                response_dict['code']=400
                response_dict['message']="Completed with errors."
        else:

                response_dict['num_files_created']=rowindex -1
                response_dict['num_csv_rows']=rowindex -1
                response_dict['code']=200
                response_dict['message']="Completed without errors."

    return response_dict

if __name__ == "__main__":


    if len(sys.argv)!=3:
        print "Usage:"
        print "csv2pjson-public [CSVFILE] [OUTPUT_DIRECTORY]"
        sys.exit(1)

    csv_file = sys.argv[1]
    output_dir = sys.argv[2]



    result = gao_csv2pjson(csv_file, output_dir)

    #output the JSON transaction summary
    print json.dumps(result, indent =4)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars - This software is public domain

import os, sys, string, json, csv, time
from collections import OrderedDict
from datetime import datetime



def new_pjson_deactive_stub():
    ps = OrderedDict()
    
    #ProviderJSON stub
    ps['enumeration_type']=""
    ps['number'] =""
    ps['title'] ="Deactive"
    ps["basic"] = OrderedDict()
    ps["basic"]["status"]= "D" 
    ps["basic"]["deactivation_date"]= ""
    return ps
    
def new_pjson_stub():
    
    ps = OrderedDict()
    
    #ProviderJSON stub
    ps['enumeration_type']=""
    ps['number'] =""
    ps['title'] = ""
    ps['last_updated_epoch'] =""
    ps['created_epoch'] =""
    ps["basic"] = OrderedDict()
    ps["basic"]["name_prefix"]= "" 
    ps["basic"]["first_name"]= "" 
    ps["basic"]["last_name"]= "" 
    ps["basic"]["middle_name"]= "" 
    ps["basic"]["name_suffix"]= "" 
    ps["basic"]["credential"]= "" 
    ps["basic"]["sole_proprietor"]= "" 
    ps["basic"]["organization_name"]= "" 
    ps["basic"]["organization_other_name"]= "" 
    ps["basic"]["organization_other_name_code"]= "" 
    ps["basic"]["organizational_subpart"]= "" 
    ps["basic"]["ssn"]= "" 
    ps["basic"]["ein"]= "" 
    ps["basic"]["itin"]= "" 
    ps["basic"]["gender"]= "" 
    ps["basic"]["date_of_birth"]= "" 
    ps["basic"]["state_of_birth"]= "" 
    ps["basic"]["country_of_birth"]= "" 
    ps["basic"]["number"]= "" 
    ps["basic"]["initial_enumeration_date"]= "" 
    ps["basic"]["enumeration_date"]= "" 
    ps["basic"]["last_updated"]= "" 
    ps["basic"]["date_of_death"]= "" 
    ps["basic"]["reactivation_date"]= "" 
    ps["basic"]["status"]= "A" 
    ps["basic"]["deactivated_details"]= "" 
    ps["basic"]["deactivation_date"]= "None" 
    ps["basic"]["deactivation_reason_code"]= "" 
    ps["basic"]["decativation_note"]= "" 
    ps["basic"]["deceased_notes"]= "" 
    ps["basic"]["parent_organization"]= "" 
    ps["basic"]["parent_organization_ein"]= "" 
    ps["basic"]["parent_organization_legal_business_name"]= "" 
    ps["basic"]["recativation_note"]= "" 
    ps["basic"]["comments"]= "" 
    ps["basic"]["authorized_official_credential"]= "" 
    ps["basic"]["authorized_official_email"]= "" 
    ps["basic"]["authorized_official_first_name"]= "" 
    ps["basic"]["authorized_official_last_name"]= "" 
    ps["basic"]["authorized_official_middle_name"]= "" 
    ps["basic"]["authorized_official_prefix"]= "" 
    ps["basic"]["authorized_official_suffix"]= "" 
    ps["basic"]["authorized_official_telephone_number"]= "" 
    ps["basic"]["authorized_official_telephone_extension"]= "" 
    ps["basic"]["authorized_official_title"]= "" 
    ps["basic"]["authorized_official_title_or_position"]= "" 
    ps["basic"]["contact_person_credential"]= "" 
    ps["basic"]["contact_person_email"]= "" 
    ps["basic"]["contact_person_first_name"]= "" 
    ps["basic"]["contact_person_last_name"]= "" 
    ps["basic"]["contact_person_middle_name"]= "" 
    ps["basic"]["contact_person_prefix"]= "" 
    ps["basic"]["contact_person_suffix"]= "" 
    ps["basic"]["contact_person_telephone_extension"]= "" 
    ps["basic"]["contact_person_telephone_number"]= "" 
    ps["basic"]["contact_person_title"]= "" 
    ps["basic"]["contact_person_title_or_position"]= ""
    ps["other_names"]       = []
    ps["addresses"]         = []
    ps["taxonomies"]        = []
    ps["licenses"]          = []
    ps["taxonomy_licenses"] = []
    ps["identifiers"]       = []
    ps["taxonomy_groups"]   = []
    return ps
    

def publiccsv2pjson(csvfile, output_dir):

    """Return a response_dict with summary of  publiccsv2pjson transaction."""


    process_start_time = time.time()
    
    pdir = 1
    
  
    #make the output dir
    try:
        os.mkdir(output_dir)
    except:
        pass
     
     
    response_dict = OrderedDict()
    fh = open(csvfile, 'rb')
    csvhandle = csv.reader(fh, delimiter=',')
    rowindex = 0
    po_count = 0
    error_list = []   
        
    for row in csvhandle :
            
        if rowindex==0:
                 
            rowindex += 1    
            column_headers = row
             
            cleaned_headers = []
            for c in column_headers:
                c= c.replace(".", "")
                c= c.replace("(", "")
                c= c.replace(")", "")
                c =c.replace("$", "-")
                c =c.replace(" ", "_")
                cleaned_headers.append(c)
        else:

            #If the records is not redacted (because its inactive)
            
            zip_record = zip(cleaned_headers, row)
            record = dict(zip(cleaned_headers, row))
            
            #get rid of blanks 
            no_blank_zip = []
            for i in zip_record:
                if i[1]:
                    no_blank_zip.append(i)
                        
            #start our object off with a stub.
            
            if row[1]:
                p = new_pjson_stub()
                
                
                if row[1] == "1":
                    p["enumeration_type"] = "NPI-1"
                    p["title"] = "%s %s" % (row[6], row[5])
                    if row[10]:
                         p["title"] = "%s, %s" % (p["title"], row[10])
                    
                elif row[1] == "2":
                    p["enumeration_type"] = "NPI-2"
                    p["title"] = row[4]
                    
                else:
                    #The record is deactivated
                    p['basic']['status'] ="D"
                    p["title"] = "Deactivated"
                
                p["number"]             = row[0] 
                #Load basic                
                
                #Provider name
                p["basic"]["name_prefix"]       = row[8].upper()
                p["basic"]['first_name']        = row[6].upper()
                p["basic"]["middle_name"]       = row[7].upper()
                p["basic"]['last_name']         = row[5].upper()
                p["basic"]["name_suffix"]       = row[9].upper()
                p["basic"]["credential"]        = row[10]
                p["basic"]["gender"]            = row[41]
                
                if row[307] in (True, "Y", "y", "YES"):
                    p["basic"]["sole_proprietor"]   = "YES"
                else:
                    p["basic"]["sole_proprietor"]   = "NO"
                    
                
                #provider_organization
                p["basic"]["organization_name"] = row[4]
                
                
                p["basic"]["ein"]               = row[3]     
                if p["basic"]["ein"] == "<UNAVAIL>":
                    p["basic"]["ein"] = ""
                
                p["basic"]["parent_organization_ein"] = row[310]
                p["basic"]["parent_organization_legal_business_name"] = row[309]
                
                
                
                p["basic"]["enumeration_date"]      = row[36]
                if p["basic"]["enumeration_date"]:
                    p['created_epoch'] = int(datetime(int(p["basic"]["enumeration_date"][6:10]),
                                                           int(p["basic"]["enumeration_date"][0:2]),
                                                           int(p["basic"]["enumeration_date"][3:5]),
                                                           0, 0).strftime('%s'))
                
                
                p["basic"]["last_updated"]              = row[37]
                if p["basic"]["last_updated"]:
                    p['last_updated_epoch'] = int(datetime(int(p["basic"]["last_updated"][6:10]),
                                                           int(p["basic"]["last_updated"][0:2]),
                                                           int(p["basic"]["last_updated"][3:5]),
                                                           0, 0).strftime('%s'))
                
                
                
                
                p["basic"]["deactivation_date"]         = row[39]
                p["basic"]["reactivation_date"]         = row[40]
                p["basic"]["deactivation_reason_code"] = row[38]
                
                if row[37]:
                    month = row[37][0:2]
                    day =  row[37][3:5]
                    year = row[37][6:10]
                    p["basic"]["last_updated"] = "%s-%s-%s" % (year, month, day)
                
                if row[36]:
                    month = row[36][0:2]
                    day =  row[36][3:5]
                    year = row[36][6:10]
                    p["basic"]["enumeration_date"] = "%s-%s-%s" % (year, month, day)
                
                if row[39]:
                    month = row[39][0:2]
                    day =  row[39][3:5]
                    year = row[39][6:10]
                    p["basic"]["deactivation_date"] = "%s-%s-%s" % (year, month, day)
                
                
                if row[40]:
                    month = row[40][0:2]
                    day =  row[40][3:5]
                    year = row[40][6:10]
                    p["basic"]["reactivation_date"] = "%s-%s-%s" % (year, month, day)
                
            
                #Contact person
                #not included in public file
               
                #Authorized official
                p["basic"]["authorized_official_last_name"] = row[42].upper()
                p["basic"]["authorized_official_first_name"] = row[43].upper()
                p["basic"]["authorized_official_middle_name"] = row[44].upper()
                p["basic"]["authorized_official_telephone_number"] = row[46].upper()
                p["basic"]["authorized_official_title_or_position"] = row[45].upper()
                
                
                clean_basic = OrderedDict()
                for k,v in p["basic"].items():
                    if v:
                        clean_basic[k] = v
                         
                p["basic"] = clean_basic
                
                if row[11] or row[13] or row[14]:
                    other_name  = OrderedDict()
                    if row[11]:
                        other_name["organization_name"]=row[11].upper()
                    
                    if row[12]:
                        other_name["code"]=row[12]
                    if row[19]:
                        other_name["code"]=row[19]
                    
                    if row[13]:
                        other_name["last_name"]=row[13].upper()
                    if row[14]:
                        other_name["first_name"]=row[14].upper()
                    if row[15]:
                        other_name["middle_name"]=row[15].upper()
                    if row[16]:
                        other_name["prefix"]=row[16].upper()
                    if row[17]:
                        other_name["suffix"]=row[17].upper()
                    if row[18]:
                        other_name["credential"]=row[18].upper()
                          
                    p["other_names"].append(other_name)    
                
                
                #Addresses
                #location ---
                a = OrderedDict()
                a["country_code"]                    = row[33]
                a["address_purpose"]                 = "LOCATION"
                
                if a["country_code"] == "US":
                    a["address_type"]                    = "DOM"
                    a["address_1"]                       =  row[28]
                    a["address_2"]                       =  row[29]
                    a["city"]                            =  row[30].upper()
                    a["state"]                           =  row[31].upper()
                    a["zip"]                             =  row[32]
                                        
                    if row[34]:
                        
                        a["us_telephone_number"]         =  "%s-%s-%s" % (row[34][0:3], row[34][3:6], row[34][6:12])
                    
                    if  row[35]:    
                        a["us_fax_number"]                   =  "%s-%s-%s" % (row[35][0:3], row[35][3:6], row[35][6:12])
    
                    
                else:
                    a["address_type"]                    = "FGN"
                    a["address_1"]                       =  row[28]
                    a["address_2"]                       =  row[29]
                    a["city"]                            =  row[30].upper()
                    a["foreign_state"]                   =  row[31].upper()
                    a["foreign_postal"]                  =  row[32]                    
                    a["foreign_telephone_number"]        =  row[34]
                    a["foreign_fax_number"]              =  row[35]
    
    
                p['addresses'].append(a)
                
                #Maiing address ---
                a = OrderedDict()
                a["country_code"]                    = row[25]
                a["address_purpose"]                 = "MAILING"
                
                if a["country_code"] == "US":
                    a["address_type"]                    = "DOM"
                    a["address_1"]                       =  row[20]
                    a["address_2"]                       =  row[21]
                    a["city"]                            =  row[22].upper()
                    a["state"]                           =  row[23].upper()
                    a["zip"]                             =  row[24]
                    
                    if row[26]:
                        a["us_telephone_number"] =  "%s-%s-%s" % (row[26][0:3], row[26][3:6], row[26][6:12])
                    
                    if row[27]:
                        
                        a["us_fax_number"]       =  "%s-%s-%s" % (row[27][0:3], row[27][3:6], row[27][6:12])
                    
                    
                    
                else:
                    a["address_type"]                    = "FGN"
                    a["address_1"]                       =  row[20]
                    a["address_2"]                       =  row[21]
                    a["city"]                            =  row[22].upper()
                    a["foreign_state"]                   =  row[23].upper()
                    a["foreign_postal"]                  =  row[24]
                    a["foreign_telephone_number"]        =  row[26]
                    a["foreign_fax_number"]              =  row[27]
    
    
                p['addresses'].append(a)
                
                #licenses and taxonomies----------------------------------------
                #starting at row 47, 2nd set starts at 51
                
                #Taxonomy
                taxonomy_code_position = 47
                primary_taxonomy_position = 50
                license_state_position = 49
                license_number_position = 48
                
                
                for i in range(1, 15):
                    
                    if row[taxonomy_code_position]:
                        taxonomy = OrderedDict()
                        taxonomy['code'] = row[taxonomy_code_position]
                        if row[primary_taxonomy_position] in ("Y", "True", True, "YES"):
                            taxonomy['primary'] = True
                        else:
                            taxonomy['primary'] = False
                        
                        p['taxonomies'].append(taxonomy)
                     
                     #License
                    if row[license_number_position]:
                        
                        license             = OrderedDict()
                        mlvs                = "%s-UNK-%s" % (row[license_state_position], row[license_number_position])
                        license['code']     = mlvs
                        license['state']    = row[license_state_position].upper()
                        license['type']     = "UNK"
                        license['status']   = "UNK"
                        
                        p['licenses'].append(license)
                    
                    #Taxonomy and License
                    if row[license_number_position] and row[taxonomy_code_position]:
                        taxonomy_license  = OrderedDict()
                        taxonomy_license['taxonomy_code'] = row[taxonomy_code_position]
                        mlvs = "%s-UNK-%s" % (row[license_state_position], row[license_number_position])
                        taxonomy_license['license_code'] = mlvs
                        p['taxonomy_licenses'].append(taxonomy_license)
                    
                    
                    taxonomy_code_position += 4
                    primary_taxonomy_position += 4
                    license_state_position += 4
                    license_number_position += 4
                
                #identifiers----------------
                #starting at row 107
                
                identifier_position = 107
                identifier_code_position = 108
                identifier_state_position = 109
                identifier_issuer_position = 110
                
                for i in range(1, 50):
                    
                    if row[identifier_position]:
                        identifier = OrderedDict()
                        identifier['identifier'] = row[identifier_position]                        
                        identifier['code'] = row[identifier_code_position]
                        identifier['state'] = row[identifier_state_position].upper()
                        identifier['issuer'] = row[identifier_issuer_position]
                        p['identifiers'].append(identifier)
                            
                
                    identifier_position        += 4
                    identifier_code_position   += 4
                    identifier_state_position  += 4
                    identifier_issuer_position += 4
                    
                #Get Taxonomy Groups for Type 2
                for i in range(314, 328):
                    if row[i]:
                        code, description = row[i].split(" ", 1)
                        tg = {"code": code, "description": description}
                        p['taxonomy_groups'].append(tg)
        
                #Remove any empty arrays
                if not p["other_names"]:
                    del p["other_names"]
                
                if not p["addresses"]:
                    del p["addresses"]
                
                if not p["taxonomies"]:
                    del p["taxonomies"]
                
                if not p["taxonomy_licenses"]:
                    del p["taxonomy_licenses"]                  
                
                if not p["licenses"]:
                    del p["licenses"]                
                     
                if not p["identifiers"]:
                    del p["identifiers"]
                    
                if not p["taxonomy_groups"]:
                    del p["taxonomy_groups"]                         
                
                
                
            else:
                
                p = new_pjson_deactive_stub()
                p["number"] = row[0]
                if row[39]:
                    month = row[39][0:2]
                    day =  row[39][3:5]
                    year = row[39][6:10]
                    p["basic"]["deactivation_date"] = "%s-%s-%s" % (year, month, day)
            
            fn = "%s.json" % (p["number"])
            
            subdir = os.path.join(output_dir, str(p["number"])[0:4])
            
            try:
                os.mkdir(subdir)
            except:
                pass
                   
                
            fp = os.path.join(subdir, fn)
            ofile =  open(fp, 'w')
            ofile.writelines(json.dumps(p, indent =4))
            ofile.close()
            po_count += 1
            if po_count % 100 == 0:
               out  = "%s files created. Total time: %s seconds." % (po_count ,(time.time() - process_start_time) )
               print out
            
            if po_count % 1000 == 0:
               pdir += 1
               out  = "%s files created. Total time: %s seconds." % (po_count ,(time.time() - process_start_time) )
               print out   
               
        
            rowindex += 1


        if error_list:
                response_dict['num_files_created']=rowindex-1
                response_dict['num_file_errors']=len(error_list)
                response_dict['errors']=error_list
                response_dict['code']=400
                response_dict['message']="Completed with errors."
        else:

                response_dict['num_files_created']=rowindex -1
                response_dict['num_csv_rows']=rowindex -1
                response_dict['code']=200
                response_dict['message']="Completed without errors."

    return response_dict

if __name__ == "__main__":

    
    if len(sys.argv)!=3:
        print "Usage:"
        print "csv2pjson-public.py [CSVFILE] [OUTPUT_DIRECTORY]"
        sys.exit(1)

    csv_file   = sys.argv[1]
    output_dir = sys.argv[2]

    result = publiccsv2pjson(csv_file, output_dir)
    
    #output the JSON transaction summary
    print json.dumps(result, indent =4)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

# Written by Alan Viars
import json, sys, datetime, re
from choices import STATES



def validate_identifier_list(l, enumeration_type):
    errors = []
    primary_count  = 0
    max_values = {
        'state'        : 2,
        'code'         : 2, 
        'issuer'       : 150, 
        'identifier'   : 20, 
        }
    for d in l:
    
        identifer_string = "%s %s %s issued by %s" % (d['identifier'],
                                            d['code'], d['state'],
                                            d['issuer'])

        
        for k in max_values.keys():
            if d.get(k):
                if max_values[k] < len(str(d.get(k))):
                    error = "%s : %s exceeds max allowable length of %s." % (identifer_string, k, max_values[k])
                    errors.append(error)
    
    
        #check for required information
        if str(d.get('code', '')) not in ("", "01", "02", "04","05", "06", "07", "08", "1", "2", "4","5", "6", "7", "8"):
            error = "%s : identifier code is not in ['', '01', '02', '04','05', '06', '07', '08']" % d.get('code')
            errors.append(error)
            
        #if state is provided then it should be valid.
        if d.get('state') and d.get('state') not in STATES:
            error = "%s : identifier code is not a valid 2-letter state code." % (identifer_string, d.get('state'))
            errors.append(error)
    
    return errors


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

# Written by Alan Viars
import json, sys, datetime, re


def validate_direct_address_list(l, enumeration_type):
    errors = []
    
    #define a max_length dict
    max_values = {'organization' : 150,
                   'email'       : 150
                 }
    
    #Test for max_length errors    
    i=0
    for d in l:
        
        for k in max_values.keys():
            if d.get(k):
                if max_values[k] < len(str(d.get(k))):
                    error = "%s : %s max allowable length %s." % (d.get('email'),  max_values[k])
                    errors.append(error)
    
        #check for required information
        if not d.get('email'):
            error = "direct_adress %s: email is required for direct ddresses." % (i)
            errors.append(error)
    
    
        if not d.get('is_public'):
            error = "direct_adress %s: is_public is required for direct addresses." % (d.get('email'))
            errors.append(error)
            
            
        if type(d.get('is_public')) != bool :
            error = "direct_adress %s: is_public must be true or false." % (d.get('email'))
            errors.append(error)
    
        if not d.get('organization'):
            error = "direct_adress %s : organization is required." % (d.get('email'))
            errors.append(error)
        i+=1
    
    
    return errors
    
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars  - This software is public domain

import json, sys

from pdt.pjson.validate_basic import validate_basic_dict
from pdt.pjson.validate_addresses import validate_address_list
from pdt.pjson.validate_licenses import validate_license_list
from pdt.pjson.validate_taxonomies import validate_taxonomy_list
from pdt.pjson.validate_identifiers import validate_identifier_list
from pdt.pjson.validate_other_names import validate_other_name_list
from pdt.pjson.validate_affiliations import validate_affiliation_list

# from validate_basic import validate_basic_dict
# from validate_addresses import validate_address_list
# from validate_licenses import validate_license_list
# from validate_taxonomies import validate_taxonomy_list
# from validate_identifiers import validate_identifier_list
# from validate_other_names import validate_other_name_list
# from validate_affiliations import validate_affiliation_list

def validate_pjson(j, action):
    """
    Input a JSON object and an action. return a list of errors. If error list
    is empty then the file is valid.
    """
    action = action.lower()

    errors =[]
    warnings =[]
    response = { "errors": [], "warnings": [] }


    #Check the action
    if action not in ("create", "update"):
        error ="action must be either create or update."
        errors.append(error)


    # Does the string contain JSON
    try:
        d = json.loads(j)
    except:
        error ="The string did not contain valid JSON."
        errors.append(error)
        response["errors"] = errors
        return response

    # Is it a dict {} (JSON object equiv)?
    if type(d)!=type({}):
        error ="The JSON string did not contain a JSON object i.e. {}."
        errors.append(error)
        response["errors"] = errors
        return response

    # Does it contain the top-level enumeration_type
    if not d.has_key("enumeration_type"):
        error ="The JSON object does not contain an enumeration_type."
        errors.append(error)
        response["errors"] = errors
        return response


    # Is the enumeration_type a valid?
    if d["enumeration_type"] not in ("NPI-1", "NPI-2", "OEID", "HPID"):
        error ="enumeration_type must be one of these: ('NPI-1', 'NPI-2', 'OEID', 'HPID')"
        errors.append(error)
        response["errors"] = errors
        return response

    #If a number is present we assume this is an update.
    if not d.has_key("number"):
        number = None
    else:
        number = d['number']

    if action=="create" and number:
        warning ="enumeration number is generated by CMS. The provided value will be ignored."
        warnings.append(warning)

    if action=="update" and not number:
        error ="enumeration number is required when performaing an update."
        errors.append(error)

    ## Sanity check epoch dates
    #if d.has_key("last_updated_epoch"):
    #    if type(d["last_updated_epoch"]) != type(1):
    #        warning ="last_updated_epoch is not an integer."
    #        warnings.append(warning)
    #else:
    #    warning ="last_updated_epoch is missing."
    #    warnings.append(warning)
    #
    #if d.has_key("created_epoch"):
    #    if type(d["created_epoch"]) != type(1):
    #        warning ="created_epoch is not an integer."
    #        warnings.append(warning)
    #else:
    #    warning ="created_date_epoch is missing."
    #    warnings.append(warning)


    #Check for errors in the basic section
    basic_errors, basic_warnings = validate_basic_dict(d.get('basic', {}),
                                                       d.get('enumeration_type'),
                                                       action, number)


    #Check for errors in the addresses
    address_errors = validate_address_list(d.get('addresses', ()), d.get('enumeration_type'))


    #Check for errors in the license section

    if d.has_key('licenses'):
        license_errors = validate_license_list(d.get('licenses', []),
                                            d.get('enumeration_type'))
    else:
        license_errors = []

    taxonomy_errors = validate_taxonomy_list(d.get('taxonomies',()),
                                             d.get('enumeration_type',()),
                        d.get('licenses', []), d.get('taxonomy_licenses', []),
                        d.get('basic', {}).get('sole_proprietor', "NO"))

    if d.has_key('identifiers'):
        identifier_errors = validate_identifier_list(d.get('identifiers',[]), d.get('enumeration_type'))
    else:
        identifier_errors = []


    if d.has_key('other_names'):
        other_names_errors = validate_other_name_list(d.get('other_names',[]),
                                                      d.get('enumeration_type'),
                                                      d.get('basic', {}))
    else:
        other_names_errors = []


    affiliation_errors, affiliation_warnings = validate_affiliation_list(d.get('affiliations',[]),
                                                      d.get('enumeration_type'))

    errors = errors + basic_errors + other_names_errors + address_errors + license_errors + \
                        taxonomy_errors + identifier_errors + affiliation_errors
    warnings = warnings + basic_warnings
    response["errors"] = errors
    response["warnings"] = warnings
    return response


if __name__ == "__main__":


    #Get the file from the command line
    if len(sys.argv)<3:
        print "You must supply a a ProviderJSON file to validate and an action"
        print "Usage: validate-pjson [ProivderJSON] [update|create]"
        sys.exit(1)
    else:
        pjson_file = sys.argv[1]
        action       = sys.argv[2]

    if action.lower() not in ("create", "update"):
        print "You must supply an action of either create or update."
        print "Usage: validate-pjson [ProivderJSON] [update|create]"
        sys.exit(1)

    #Try to open the file
    try:
        fh = open(pjson_file, 'r')

        j = fh.read()

        #Validate the provider JSON content
        errors = validate_pjson(j, action)
        #Print the erros and warings as JSON to stout.
        errors_json =  json.dumps(errors, indent =4)
        print errors_json
    except IOError:
        print "Could not open file %s." % (pjson_file)
        sys.exit(1)







#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars

import json, sys, datetime, re
from choices import COUNTRIES, STATES
MIN_AGE_IN_DAYS = 6570

def validate_basic_dict(d, enumeration_type, action, number=None):
    """
    Input a python dict(d) object. Return a list of errors. If error list
    is empty then basic section is valid.
    """
    errors =[]
    warnings = []
    print "action", action
     #check values do not exceed max length

    max_values ={
                'name_prefix'                 : 5,
                'first_name'                  : 150,
                'last_name'                   : 150,
                'middle_name'                 : 150,
                'name_sufix'                  : 5,
                'credential'                  : 50,
                'doing_business_as'           : 300,
                'sole_proprieter'             : 3,
                'organization_name'           : 300,
                'organization_other_name'     : 300,
                'organization_other_name_code': 1,
                'ssn'                         : 9,
                'ein'                         : 9,
                'itin'                        : 9,
                'gender'                      : 1,
                'state_of_birth'              : 2,
                'country_of_birth'            : 2,
                'mode'                        : 1,
                'status'                      : 1,
                'contact_method'              : 1,
                'classification'              : 1,
                'deactivated_details'         : 1024,
                'deactivation_reason_code'    : 2,
                'deceased_notes'              : 1024,
                'parent_organization_npi'     : 9,
                'parent_organization_ein'     : 9,
                'parent_organization_legal_business_name': 300,
                'reactivation_note'           : 1024,
                'comments'                    : 1024,
                'contact_person_credential'   : 20,
                'contact_person_email'        : 75,
                'contact_person_first_name'   : 150,
                'contact_person_last_name'    : 150,
                'contact_person_middle_name'  : 150,
                'contact_person_prefix'       : 5,
                'contact_person_suffix'       : 4,
                'contact_person_telephone_extension'        : 10,
                'contact_person_telephone_number'           : 12,
                'contact_person_title_or_position'          : 150,
                'authorized_official_credential'            : 50,
                'authorized_official_email'                 : 75,
                'authorized_official_first_name'            : 300,
                'authorized_official_last_name'             : 300,
                'authorized_official_middle_name'           : 300,
                'authorized_official_prefix'                : 4,
                'authorized_official_suffix'                : 5,
                'authorized_official_telephone_number'      : 12,
                'authorized_official_telephone_extension'   : 12,
                'authorized_official_title_or_position'     : 300,
                'website'                                   : 1024,
                'gravatar_email'                            : 200,
                'facebook_handle'                           : 100,
                'twitter_handle'                            : 100,
                'public_email'                              : 75,
                'driving_directions'                        : 1024,
                'bio_headline'                              : 256,
                }

    for k in max_values.keys():
        if d.get(k):
            if max_values[k] < len(str(d.get(k))):
                error = "%s exceeds max allowable length of %s." % (k, max_values[k])
                errors.append(error)

    #Validate Common items ------------------------------------------


    if d.get('enumeration_date'):
        try:
            date = datetime.datetime.strptime(d.get('enumeration_date'), '%Y-%m-%d').date()
        except ValueError:
            error = "enumeration_date must be in YYYY-MM-DD format."
            errors.append(error)
        warning = "Enumeration date is generated by CMS. The provided value will be ignored."
        warnings.append(warning)

    if d.get('last_updated'):
        try:
            date = datetime.datetime.strptime(d.get('last_updated'), '%Y-%m-%d').date()
        except ValueError:
            error = "last_updated must be in YYYY-MM-DD format."
            errors.append(error)
        warning = "Last updated date is generated by CMS. The provided value will be ignored."
        warnings.append(warning)

    if d.get('initial_enumeration_date'):
        try:
            date = datetime.datetime.strptime(d.get('initial_enumeration_date'), '%Y-%m-%d').date()
        except ValueError:
            error = "initial_enumeration_date must be in YYYY-MM-DD format."
            errors.append(error)
        warning = "Initial enumeration date is generated by CMS. The provided value will be ignored."
        warnings.append(warning)

    if d.get('date_of_death'):
        try:
            date = datetime.datetime.strptime(d.get('date_of_death'), '%Y-%m-%d').date()
        except ValueError:
            error = "date_of_death must be in YYYY-MM-DD format."
            errors.append(error)
        warning = "Date of death is determined by the SSA. The provided value will be ignored."
        warnings.append(warning)


    if d.get('reactivation_date'):
        try:
            date = datetime.datetime.strptime(d.get('reactivation_date'), '%Y-%m-%d').date()
        except ValueError:
            error = "reactivation_date must be in YYYY-MM-DD format."
            errors.append(error)
        warning = "Reactivation date is generated by CMS. the provided value will be ignored."
        warnings.append(warning)

    if d.get('deactivation_date'):
        try:
            date = datetime.datetime.strptime(d.get('deactivation_date'), '%Y-%m-%d').date()
        except ValueError:
            error = "deactivation_date must be in YYYY-MM-DD format."
            errors.append(error)
        warning = "Deactivation date is generated by CMS. The provided value will be ignored."
        warnings.append(warning)

    #validate phone numbers

    if d.get('contact_person_telephone_number') and not re.match(r'^[0-9]{3}-[0-9]{3}-[0-9]{4}$',d.get('contact_person_telephone_number')):
        error = "contact_person_telephone_number must be in XXX-XXX-XXXX format."
        errors.append(error)

    if d.get('authorized_official_telephone_number') and not re.match(r'^[0-9]{3}-[0-9]{3}-[0-9]{4}$',d.get('authorized_official_telephone_number')):
        error = "authorized_official_telephone_number must be in XXX-XXX-XXXX format."
        errors.append(error)

    #Meta fields -----------------------------------
    if d.get("mode") and  d.get("mode", "").upper() not in ('W', 'P', 'E', 'A'):
        #Note this should always be (E)lectronic if submitting via API
        error = "mode must be in ('W','P','E', 'A')."
        errors.append(error)
    
    if d.get("mode") and d.get("mode") != 'A':
        #Note this should always be (E)lectronic if submitting via API
        warning = "The mode should be set to A when using the API."
        warnings.append(warning)



    if d.get("status") and d.get("status").upper() not in ('E', 'P', 'A', 'D', 'R'):

        #Note: NPPES will ignore this since status is decided by NPPES.
        error = "status must be in ('E', 'P', 'A', 'D', 'R')."
        errors.append(error)
    
    if d.get("status"):
        
        warning = "status is determined by CMS. The provided value will be ignored."
        warnings.append(warning)

    if d.get("contact_method") and d.get("contact_method") not in ('M', 'E',):

        #Note: NPPES will ignore this since status is decided by NPPES.
        error = "contact_method must be in ('M', 'E')."
        errors.append(error)


    # NPI-1 -------------------------------------------------------------------
    if enumeration_type == "NPI-1":
        #Ensure required fields for NPI-1

        if not d.get('first_name'):
            error = "first_name is required."
            errors.append(error)

        if not d.get('last_name'):
            error = "last_name is required."
            errors.append(error)


        if d.get("name_suffix") and d.get("name_suffix").upper()  not in ('JR.', 'SR.', 'I', 'II', 'III', 'IV',
                                        'V', 'VI', 'VII', 'VIII', 'IX', 'X'):
            error = """name_suffix must be in ['Jr.', 'Sr.', 'I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', 'IX', 'X']'
            """
            errors.append(error)

        if not d.get('sole_proprietor'):
            error = "sole_proprietor is required and must be in ('YES', 'NO')."
            errors.append(error)
        else:
            if d.get('sole_proprietor').upper() not in ("YES","NO"):
                error = "sole_proprietor must be in ('YES', 'NO')."
                errors.append(error)
        #Gender
        if action=="create" and not d.get('gender'):
            error = "gender is required."
            errors.append(error)
       
        elif action=="create" and d.get('gender'):
            if d.get('gender') not in ("M","F", "T"):
                error = "gender must be in ('F','M', 'T')."
                errors.append(error)
        
        elif action=="update" and d.get('gender'):
            warning = "gender cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning) 
            
        #Date of Birth
        if action=="create" and not d.get('date_of_birth'):
            error = "date_of_birth is required when creating a new individual's record."
            errors.append(error)
            
        elif action=="update" and d.get('date_of_birth'):
            warning = "date_of_birth cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning) 
            
        elif action=="create" and d.get('date_of_birth'):
            # date supplied so let's make sure it is valid
            try:
                date_of_birth = datetime.datetime.strptime(d.get('date_of_birth'), '%Y-%m-%d').date()
                
                age_days =  datetime.date.today() - date_of_birth
                if age_days.days < MIN_AGE_IN_DAYS:
                    errors.append("Candidate must be at least 16 years of age.")
                
            except ValueError:
                error = "date_of_birth must be in YYYY-MM-DD format."
                errors.append(error)

        #State of Birth
        if action=="create" and not d.get('state_of_birth'):
            error = "state_of_birth is required. Use ZZ if born outside the US."
            errors.append(error)
        elif action=="create" and d.get('state_of_birth'):
            if d.get('state_of_birth') not in STATES:
                error = "state_of_birth must be 2 letter ISO code or ZZ for foreign born."
                errors.append(error)
        elif action=="update" and d.get('state_of_birth'):
            warning = "state_of_birth cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning)    
        
        
        #Country of Birth
        if action=="create" and not d.get('country_of_birth'):
            error = "country_of_birth is required for NPI-1 creation.."
            errors.append(error)
        elif action=="create" and d.get('country_of_birth'):
             if d.get('country_of_birth') not in COUNTRIES:
                error = "country_of_birth must be 2 letter ISO code."
                errors.append(error)
        elif action=="update" and d.get('country_of_birth'):
            warning = "country_of_birth cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning)


        # Validate the interdependecies
        if d.get('country_of_birth') and d.get('country_of_birth') and \
        (d.get('country_of_birth')) != "US" and (d.get('state_of_birth') != "ZZ"):
            error = """country_of_birth and state_of_birth mismatch. A person cannot be born in both a foreign contry and a US state at the same time."""
            errors.append(error)

        if action == "create" and not d.get('ssn') and not d.get('itin'):
            error = "An NPI-1 individual provider must supppy either an SSN or an EIN."
            errors.append(error)
        
        if action == "update" and d.get('ssn'):
            warning = "Social security number (ssn) cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning)

        if action == "update" and d.get('itin'):
            warning = "Individual Taxpayer Identification Number (itin) cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning)


        if d.get('ssn') and len(str(d.get('ssn'))) != 9 :
            error = "SSN must be 9 digits."
            errors.append(error)

        if d.get('itin') and len(str(d.get('itin'))) != 9 :
            error = "ITIN must be 9 digits."
            errors.append(error)


        # ensure a contact person is given
        if not d.get('contact_person_email'):
            error = "contact_person_email must be provided."
            errors.append(error)

        # ensure a contact person is given
        if not d.get('contact_person_first_name'):
            error = "contact_person_first_name must be provided."
            errors.append(error)

        # ensure a contact person is given
        if not d.get('contact_person_last_name'):
            error = "contact_person_last_name must be provided."
            errors.append(error)

        # ensure a contact person is given
        if not d.get('contact_person_telephone_number'):
            error = "contact_person_telephone_number must be provided."
            errors.append(error)

        #Validate the not required items NPI-1

        if d.get("name_prefix") and d.get("name_prefix").upper() not in ('MS.', 'MR.', 'MISS',
                                                                         'MRS.', 'DR.', 'PROF.'):
                error = "name_prefix  must be one of the following: 'Ms.', 'Mr.', 'Miss', 'Mrs.', 'Dr.', 'Prof.'"
                errors.append(error)


    if enumeration_type == "NPI-2":
        #Validate the organization
        if not d.get('organization_name', ""):
            error = "organization_name is required for a type 2 (organization) NPI."
            errors.append(error)
        else:
            if len(d.get('organization_name')) > 300:
                error = "organization_name is longer than allowed."
                errors.append(error)

        if not d.get('ein'):
            error = "EIN is required for a type-2 organization provider."
            errors.append(error)

        if d.get('ein') and len(str(d.get('ein'))) != 9 :
            error = "EIN must be 9 digits."
            errors.append(error)

        #if not d.get('authorized_official_email'):
        #    error = "authorized_official_email is required for a type-2 organization provider."
        #    errors.append(error)

        if not d.get('authorized_official_first_name'):
            error = "authorized_official_first_name is required for a type-2 organization provider."
            errors.append(error)

        if not d.get('authorized_official_last_name'):
            error = "authorized_official_last_name is required for a type-2 organization provider."
            errors.append(error)
            
        if not d.get('authorized_official_title_or_position'):
            error = "authorized_official_title_or_position is required for a type-2 organization provider."
            errors.append(error)

        if not d.get('authorized_official_telephone_number'):
            error = "authorized_official_telephone_number is required for a type-2 organization provider."
            errors.append(error)
    retval = [errors, warnings]
    return retval



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

# Written by Alan Viars
import json, sys, datetime, re
from choices import (SUFFIX_CHOICES , PREFIX_CHOICES, INDIVIDUAL_OTHER_NAME_CHOICES,
                     SOLE_PROPRIETOR_OTHER_NAME_CHOICES, 
                     ORGANIZATION_OTHER_NAME_CHOICES,
                     OTHER_NAME_CHOICES)
                    




def validate_other_name_list(l, enumeration_type, basic):
    errors = []
    primary_count  = 0
    
    if enumeration_type == "NPI-1":
        sole_proprietor = basic['sole_proprietor']
    else:
        sole_proprietor =  "NO"
        

    max_values = {
        'type'              : 100,
        'code'              : 1,
        'prefix'            : 5,
        'organization_name' : 150,
        'first_name'        : 150,
        'middle_name'       : 150,
        'last_name'         : 150,
        'suffix'            : 4,
        'credential'        : 50,
        'othertype'         : ''
        }
    i=0
    for d in l:
    
        for k in max_values.keys():
            if d.get(k):
                if max_values[k] < len(str(d.get(k))):
                    error = "other_name %s: %s exceeds max allowable length of %s." % (i, k, max_values[k])
                    errors.append(error)
        
        # Type II
        if enumeration_type == "NPI-2" and str(d.get('code', '')) not in ORGANIZATION_OTHER_NAME_CHOICES:
            error = "other_name %s: %s other_name for organizations is not in %s." % (i, d.get('code'), ORGANIZATION_OTHER_NAME_CHOICES)
            errors.append(error)
            
        if enumeration_type == "NPI-2" and not str(d.get('organization_name', '')):   
            error = "other_name %s: organization_name required for NPI-2," % (i)
            errors.append(error)
  
        #Type 1
        if enumeration_type == "NPI-1" and sole_proprietor ==  "NO" and str(d.get('code', '')) not in INDIVIDUAL_OTHER_NAME_CHOICES:
            error = "other_name %s: %s other_name for individuals is not in %s" % (i, d.get('code'), INDIVIDUAL_OTHER_NAME_CHOICES)
            errors.append(error)
            
            
        if enumeration_type == "NPI-1" and sole_proprietor ==  "NO" and not str(d.get('first_name', '')):   
            error = "other_name %s: first_name required for NPI-1." % (i)
            errors.append(error)
            

        if enumeration_type == "NPI-1" and sole_proprietor ==  "NO" and not str(d.get('last_name', '')):   
            error = "other_name %s: last_name required for NPI-2" % (i)
            errors.append(error)
        
        #Type 1 Sole proprietor
        if enumeration_type == "NPI-1" and sole_proprietor ==  "YES" and str(d.get('code', '')) not in SOLE_PROPRIETOR_OTHER_NAME_CHOICES:
            error = "other_name %s: %s other_name for sole_proprietor is not in %s." % (i, d.get('code'), SOLE_PROPRIETOR_OTHER_NAME_CHOICES)
            errors.append(error)
            
        if enumeration_type == "NPI-1" and sole_proprietor ==  "YES" \
            and not ( str(d.get('last_name', '')) and str(d.get('first_name', ''))) \
            and not str(d.get('organization_name', '')):
              error = "other_name %s: Either organization_name or first_name and last_name are required for a sole proprietor." % (i)
              errors.append(error)          
        
        #General
        #supply othertypwe when type is not given.
        if not str(d.get('code', '')) and not str(d.get('othertype', '')):
            error = "other_name %s: othertype must be supplied if other_name type code is blank." % (i)
            errors.append(error)
    
        #suffix
        suffix = str(d.get('suffix', '')).upper()
        if suffix and suffix not in SUFFIX_CHOICES:
            error = "other_name %s:  suffix must be in %s" % (i, SUFFIX_CHOICES)
            errors.append(error)
            
        #prefix
        prefix = str(d.get('prefix', '')).upper()
        if prefix and prefix not in PREFIX_CHOICES:
            error = "other_name %s:  prefix must be in %s" % (i, PREFIX_CHOICES)
            errors.append(error)
            
            

        i+=1
    return errors


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars

import json, sys, os, datetime, csv
from choices import TAXONOMY_CODES


def validate_taxonomy_list(taxonomies, enumeration_type, licenses,
                           taxonomy_licenses, sole_proprietor):
    
    csvfile = os.path.join( os.path.dirname( __file__),  "taxonomy-license-crosswalk.csv")
    
    errors = []
    primary_count  = 0

    for t in taxonomies:
 
        #check for required information
        if t.get('code') not in TAXONOMY_CODES:
            error = "%s : code (taxonomy) is not a valid taxonomy code. See http://www.wpc-edi.com/taxonomy" % t.get('code')
            errors.append(error)

        if type(t.get('primary')) != type(True) :
            error = "%s : primay must be true or false." % (d.get('code'))
            errors.append(error)

        if t.get('primary') == True:
            primary_count += 1
    # check that only one taxonomy is marked as primary

    if primary_count != 1:
        error = "Exactly 1 taxonomy code must be marked as primary. The primary count is %s." % (primary_count)
        errors.append(error)
    
    taxonomy_license_crosswalk ={}
    #Load the CSV
    fh = open(csvfile, 'rb')
    csvr = csv.reader(fh, delimiter=',')
    for row in csvr:
        key = row[0]
        entity_type = row[1]
        if row[2] =="False":
            license_required = False
        else:
            license_required = True
        
        taxonomy_license_crosswalk[key] =  (entity_type, license_required  ) 
    
    #Get the taxonomy license codes  
    tl_codes = []
    for tl in taxonomy_licenses: 
        tl_codes.append(tl.get('taxonomy_code')) 
    
    for t in taxonomies:
        #print t
        #check for required information
        
        #Check to see if the taxonomy is right for the enumeration_type
        if t.get('code') in taxonomy_license_crosswalk.keys():
            cw = taxonomy_license_crosswalk[t.get('code')]
            code = cw[0]
            requires_license = cw[1]
            
            #print code , requires_license
            
            if cw[0]=="I" and enumeration_type != "NPI-1" and sole_proprietor=="NO":
                error = "Taxonomy %s is only for for individuals / NPI-1." % (t.get('code'))
                errors.append(error)
            if cw[0]=="O" and enumeration_type != "NPI-2":
                error = "Taxonomy %s is only for for organizations / NPI-2." % (t.get('code'))
                errors.append(error)
            #Check to see if the taxonomy requires a license     
            if requires_license and t.get('code') not in tl_codes:
                error = "Taxonomy %s requires a license." % (t.get('code'))
                errors.append(error)
                
    return errors




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars - This software is public domain

from bs4 import BeautifulSoup
import urllib2
import re, glob, sys
from subprocess import call

def do_update(process_full=True, download=True):
    months = ["January","Feburary","March","April", "May", "June"]
    #Get just the html page
    html_page = urllib2.urlopen("http://nppes.viva-it.com/NPI_Files.html")
    soup = BeautifulSoup(html_page)
    month =""
    year="2015"
    #get all links
    zipfilelinks = []
    for link in soup.findAll('a'):
        #print link
        #get just zips
        if link.get('href', "").endswith(".zip"):
            zipfilelinks.append(link.get('href', ""))
    
    #determine full v/s weekly
    weeklylinks = []
    full_link =""
    for link in zipfilelinks:

        for m in months:
            if link.__contains__(m):
                full_link = link
                month = m
        if link.__contains__("Report"):
            deactivation_link = link
        else:
            weeklylinks.append(link)
                    
            
    #Download full file
    if process_full:
        print 
        if download:
            print "Downloading", full_link        
            call(["wget", full_link])
        
        #Get filename and unzip
        filename = full_link.split("/")
        zipfilename = filename[-1]
        print "Unzip", zipfilename
       
        call(["unzip",zipfilename])
        
        #inspect the local directory for the CSV
        csv_files = glob.glob("*.csv")
        
        for f in csv_files:
            if f.__contains__("Header"):
                header_file = f
            else:
                main_file_to_import = f
        
        #Now import the file
        print "Import", main_file_to_import
        #first convert to json
        json_output_dir = "json-output"
        call(["csv2pjson-public.py", main_file_to_import, json_output_dir ])
        #now upload to mongo
        call(["jsondir2mongo", json_output_dir, "nppes", "pjson", "T", "127.0.0.1", "27017" ])
    
    
        #now create indexes
        call(["create-provider-indexes", "nppes", "pjson", "127.0.0.1", "27017", "Y" ])
    #Download weekly files
        #for link in weeklylinks:
        #    print "weekly", link        


if __name__ == "__main__":
    
    
    if len(sys.argv)!=3:
        print "Usage:"
        print "loadnppes.py [PROCESS_FULL Y/N] [DOWNLOAD_FROM_PUBLIC_FILE Y/N]"
        print "Example:"
        print "loadnppes.py Y Y"
        sys.exit(1)

    if sys.argv[1] in ("Y", "y", "T", True):
        process_full = True
    else:
        process_full = False
    
    if sys.argv[2] in ("Y", "y", "T", True):
        download = True
    else:
        download = False
    
    #Get the file from the command line
    do_update(process_full, download)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars

import json, sys, datetime, re
from choices import AFFILIATION_DATA_TYPE, AFFILIATION_PURPOSE, ENDPOINT_DATA_TYPE, STATES
from validate_email import validate_email


def validate_affiliation_list(l, enumeration_type):
    errors = []
    warnings = []
    primary_count  = 0
    max_values = {
        'purpose': 25,
        'affiliation_data_type': 10,
        'endpoint_data_type': 50,
        'affiliation_identifier': 1024,
        'endpoint': 1024,
        'description': 1024,
        'state': 2,
        }

    i=0

    for d in l:

        affiliation_string = "Affiliation %s: " % (i)

        for k in max_values.keys():
            if d.get(k):
                if max_values[k] < len(str(d.get(k))):
                    error = "%s : %s exceeds max allowable length of %s." % (affiliation_string,
                                                                  k,
                                                                  max_values[k])
                    errors.append(error)


        #check for required information
        if d.get('purpose') not in AFFILIATION_PURPOSE:
            error = "%s purpose %s is not a valid value.  Valid values are %s." % (
                affiliation_string, d.get('purpose'), AFFILIATION_PURPOSE)
            errors.append(error)


        if d.get('affiliation_data_type') not in AFFILIATION_DATA_TYPE:
            error = "%s affiliation_data_type %s is not a valid value.  Valid values are %s." % (
                                            affiliation_string,
                                            d.get('affiliation_data_type'),
                                            AFFILIATION_DATA_TYPE)
            errors.append(error)

        if not str(d.get('affiliation_data_type')) and \
                d.get('purpose') not in AFFILIATION_PURPOSE:
            error = "%s affiliation_data_type %s is required when purpose is one of these %s" % (
                                                    affiliation_string,
                                                    d.get('affiliation_data_type'),
                                                    AFFILIATION_PURPOSE)
            errors.append(error)

        if d.get('endpoint_data_type') and d.get('endpoint_data_type') not in ENDPOINT_DATA_TYPE:
            error = "%s endpoint_data_type %s is not a valid value. Valid values are %s." % \
                  (affiliation_string, d.get('endpoint_data_type', ''), ENDPOINT_DATA_TYPE)
            errors.append(error)

        if d.get('purpose', '') == "HIE-EXCHANGE" and not d.get('endpoint_data_type', ''):
            error = "%s endpoint_data_type is required when the purpose is HIE-EXCHANGE." % \
                  (affiliation_string)
            errors.append(error)

        if not d.get('affiliation_identifier', '') and \
            d.get('affiliation_data_type', '') in AFFILIATION_DATA_TYPE:
            error = "%s affiliation_identifier is required when affiliation_data_type is in %s." % \
                  (affiliation_string, AFFILIATION_DATA_TYPE)
            errors.append(error)

        if not d.get('endpoint') and d.get('purpose') in ("HIE-EXCHANGE", "DOMAIN"):
            error = "%s endpoint is required when purpose is in %s." % \
                  (affiliation_string, ("HIE-EXCHANGE", "DOMAIN"))
            errors.append(error)

        if d.get('accepting_new_patients', None) not in (True, False, None):
            error = "%s accepting_new_patients must be boolean. i.e. true or false." % \
                  (affiliation_string)
            errors.append(error)

        if d.get('for_additional_documentation_request', None) not in (True, False, None):
            error = "%s for_additional_documentation_request must be boolean. i.e. true or false." % \
                  (affiliation_string)
            errors.append(error)

        if d.get('purpose') == "MEDICAID-NETWORK" and not  d.get('state'):
            error = "%s state is required when purpose = MEDICIAD-NETWORK." % \
                  (affiliation_string)
            errors.append(error)

        if d.get('state') and d.get('state') not in STATES:
            error = "%s state %s is not a valid value. Valid values are %s." % \
                  (affiliation_string, d.get('state'), STATES)
            errors.append(error)

        #if d.get('endpoint_data_type') in ('DIRECT-EMAIL-ADDRESS', 'REGULAR-EMAIL-ADDRESS'):
        #    is_valid = validate_email(d.get('endpoint'))
        #    if not is_valid:
        #        error = "%s %s has and endpoint_data_type of %s and is not a valid email." % \
        #              (affiliation_string, d.get('endpoint'), d.get('endpoint_data_type') )
        #        errors.append(error)

        i += 1
    retval = [errors, warnings]
    return retval


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

# Written by Alan Viars
import json, sys, datetime, re
from choices import LICENSE_TYPE_CODES

def validate_license_list(l, enumeration_type):
    errors = []

    #define a max_values dict
    max_values = {
        'code'              : 50,
        'status'           : 10,
        }

    #Test for max_length errors

    for d in l:
        # form an address string.

        for k in max_values.keys():
            if d.get(k):
                if max_values[k] < len(str(d.get(k))):
                    error = "%s : %s max allowable length %s." % (d.get['code'], max_values[k])
                    errors.append(error)

        if len(d.get('code')) < 7:
            #The code is to shourt to be valid
            error = "%s : License code is too short to be valid." % (d.get('code'))
            errors.append(error)
        else:
            license_code  = d.get('code')[0:6]
            if license_code not in LICENSE_TYPE_CODES:
                error = "%s : Licese code must be a valid license type code.  See https://github.com/HHSIDEAlab/mlvs/blob/master/docs/USProviderLicenseTypesFeb2014.csv" % (d.get('code'))
                errors.append(error)

    return errors


